<?php

// mail_digest.php
// Written by Mark D. Hamill, mhamill@computer.org
// This software is designed to work with phpBB Version 2.0.6

// ----------------------------------------- WARNING ---------------------------------------------- //
// THIS PROGRAM SHOULD BE INVOKED TO RUN AUTOMATICALLY EVERY HOUR BY THE OPERATING SYSTEM USING AN 
// OPERATING SYSTEM FEATURE LIKE CRONTAB. SEE BATCH_SCHEDULING.TXT!!!
// ----------------------------------------- WARNING ---------------------------------------------- //

// Warning: this was only tested with MySQL. I don't have access to other databases. Consequently, 
// the SQL may need tweaking for other relational databases.

define('IN_PHPBB', true);
$phpbb_root_path = './'; 
include($phpbb_root_path . 'extension.inc');
include($phpbb_root_path . 'common.' . $phpEx);
include($phpbb_root_path . 'includes/digest_emailer.'.$phpEx);
include($phpbb_root_path . 'includes/digest_constants.' . $phpEx);

$link_tag="";
$link_tag_unset = true;

// Is today the day to run the weekly digest?
$today = getdate();
$mday = $today['wday'];
$current_hour = $today['hours'];
$weekly_digest_text = ($mday == $weekly_digest_day ) ? " or (digest_type = 'WEEK' and send_hour = " . $current_hour . ")" : "";

// Send a user a weekly digest only if it is the correct day and hour of the week for a weekly 
// digest, and any daily digest if the current hour of the day is the same as the hour wanted for the digest. 

$sql = "select s.user_id, u.username, u.user_email, u.user_lastvisit, 
        from_unixtime(u.user_lastvisit, \"%d %b %Y %h:%m %p\")  as 'Last Visited',
        digest_type, format, show_text, show_mine, new_only, send_on_no_messages, send_hour 
        from " . SUBSCRIPTIONS_TABLE . " s, " . USERS_TABLE . " u
        where s.user_id = u.user_id and ((digest_type = 'DAY' and send_hour = " . $current_hour . 
        ")" . $weekly_digest_text . ")";

if ( !($result = $db->sql_query($sql)))
  {
    message_die(GENERAL_ERROR, 'Unable to retrieve list of users wanting a digest', '', __LINE__, __FILE__, $sql);
  }

// Retrieve a list of forum_ids that all registered users can access. Since digests go only to registered
// users it's important to include those forums not accessible to the general public but accessible to users.
$sql2 = "select forum_id from " . FORUMS_TABLE . " where auth_read in (" . AUTH_ALL . ", " . AUTH_REG . ")";

if ( !($result2 = $db->sql_query($sql2)))
  {
    message_die(GENERAL_ERROR, 'Unable to retrieve list of forum_ids all members can access', '', __LINE__, __FILE__, $sql);
  }
$i = 0;
while ($row2 = $db->sql_fetchrow($result2)) {
   $valid_forums [$i] = $row2['forum_id'];
   $i++;
}

$db->sql_freeresult($result2);

// With each pass through the loop one user will receive a customized digest.

while ($row = $db->sql_fetchrow($result)) {

   if ($row['new_only'] == "TRUE") {

     // To filter out any possible messages a user may have seen we need to examine a number of 
     // possibilities, including last user message date/time, date/time of last session, if it exists, and
     // of course, the last access date/time in the USERS table. Of these 3 possibilities, whichever is
     // the greatest value is the actual last accessed date, and we may need to filter out messages
     // prior to this date and time. My experience is phpBB doesn't always get it right.

     $sql3 = "select max(post_time) as last_post_date from " . POSTS_TABLE . 
              " where poster_id = " . $row['user_id'];

     if ( !($result3 = $db->sql_query($sql3)))
       {
         message_die(GENERAL_ERROR, 'Unable to select last post date for user.', '', __LINE__, __FILE__, $sql);
       }
     $row3 = $db->sql_fetchrow($result3);
     $last_post_date = ($row3['last_post_date'] == "") ? $row3['last_post_date'] : 0;
     $db->sql_freeresult($result3);

     // When did the user's last session accessed?
     $sql3 = "select max(session_time) as last_session_date 
              from " . SESSIONS_TABLE .
              " where session_user_id = " . $row['user_id'];

     if ( !($result3 = $db->sql_query($sql3)))
       {
         message_die(GENERAL_ERROR, 'Unable to get last session date for user', '', __LINE__, __FILE__, $sql);
       }
     $row3 = $db->sql_fetchrow($result3);
     $last_session_date = ($row3['last_session_date'] == "") ? $row3['last_session_date'] : 0;
     $db->sql_freeresult($result3);

     $last_visited_date = $row['user_lastvisit'];
     if ($last_visited_date == "")
        $last_visited_date = 0;

     // The true last visit date is the greatest of: last_visited_date, last message posted, and last session date
     $last_visited_date = max($last_post_date, $last_session_date, $last_visited_date);

   }

   // Get a list of forums that can only be read if user has been granted explicit permission
   $i=0;
   $elected_forums = array();
   $sql3 = "select distinct a.forum_id
            from " . AUTH_ACCESS_TABLE . " a, " . USER_GROUP_TABLE . " ug
            where ug.user_id = " . $row['user_id']
            . " and ug.user_pending = 0 
            and a.group_id = ug.group_id";

   if ( !($result3 = $db->sql_query($sql3)))
     {
       message_die(GENERAL_ERROR, 'Unable to retrieve list of elected forums', '', __LINE__, __FILE__, $sql);
     }
   while ($row3 = $db->sql_fetchrow($result3)) {
      $elected_forums [$i] = $row3['forum_id'];
      $i++; 
   }
   $db->sql_freeresult($result3);

   // Get the union of the valid_forums array and the elected_formus array. The resulting elements are 
   // the forums that could be queried. This is necessary because MySQL doesn't support SQL Unions (yet).
   $queried_forums = array_merge($valid_forums, $elected_forums);
   $queried_forums = array_unique($queried_forums);

   // Further filter the number of messages sent by including only forums from which the user
   // specifically wants to get emails. If there are none, assume all.
   $i=0;
   $subscribed_forums = "";
   $sql3 = "SELECT forum_id
            from " . SUBSCRIBED_FORUMS_TABLE .
            " where user_id = " . $row['user_id'];

   if ( !($result3 = $db->sql_query($sql3)))
     {
       message_die(GENERAL_ERROR, 'Unable to retrieve list of subscribed forums', '', __LINE__, __FILE__, $sql);
     }

   while ($row3 = $db->sql_fetchrow($result3)) {
      $subscribed_forums [$i] = $row3['forum_id'];
      $i++; 
   }

   $db->sql_freeresult($result3);

   // If there are subscribed forums, we only want to see messages for these forums.
   if ($i <> 0)  
      $queried_forums = array_intersect($queried_forums, $subscribed_forums);

   // Create a list of forums to be queried from the database. This is a comma delimited list of all forums
   // the user is allowed to read that can be used with the SQL IN operation.
   $forum_list = implode(",",$queried_forums);

   // Format sender's email address (SMTP seems to have a problem with adding username)
   $to = ($board_config['smtp_delivery']) ? $row['user_email'] : $row['username'] . " <" . $row['user_email'] . ">";

   // Show the text of the message?
   $show_text = ($row['show_text'] == "YES") ? true: false; 

   // Show messages written by this user?
   $show_mine = ($row['show_mine'] == "YES") ? true: false; 

   // Prepare to get digest type
   if($row['digest_type'] == "DAY") {
      $msg_period = $lang['digest_period_24_hrs'];
      $period = "1 DAY";
   }
   else {
      $msg_period = $lang['digest_period_1_week'];
      $period = "7 DAY";
   }

   // Format differently if HTML requests
   if($row['format'] == "HTML") { 
      $html = true;
      $parastart = "<p>";
      $paraend = "</p>\r\n";
   }
   else {
      $html = false;
      $parastart = "";
      $paraend = "\r\n\r\n";
   }

   // Set part of SQL needed to retrieve new only, or messages through the selected period
   if ($row['new_only'] == "TRUE") 
      $code = "GREATEST(UNIX_TIMESTAMP(DATE_SUB(CURRENT_DATE, INTERVAL " . $period . ")), " . $last_visited_date . ")";         
   else 
      $code = "UNIX_TIMESTAMP(DATE_SUB(CURRENT_DATE, INTERVAL " . $period . "))"; 

   // Filter out user's own postings, if they so elected
   if ($show_mine == false)
      $code .= " and p.poster_id <> " . $row['user_id'];

   // The emailer class does not have the equivalent of the assign_block_vars operation, so the
   // entire digest must be placed inside a variable.
   $msg = "";

   // Create a list of messages for this user that presumably have not been seen.
   // Filter out unauthorized forums.
   $sql2 = "select c.cat_title, f.forum_name, t.topic_title, u.username as \"Posted by\", 
      post_time, from_unixtime(post_time, \"%d %b %Y, %h:%m %p\") as \"Display Time\",
      if(length(pt.post_text)<=" . $extract_size . ",pt.post_text,concat(substring(pt.post_text,1," . $extract_size. ") ,\"...\")) as \"Post Text\", p.post_id, t.topic_id, f.forum_id
      from " . POSTS_TABLE . " p, " . TOPICS_TABLE . " t, " . FORUMS_TABLE . " f, " . USERS_TABLE . " u, " .
      CATEGORIES_TABLE . " c, " . POSTS_TEXT_TABLE . " pt
      where p.topic_id = t.topic_id and t.forum_id = f.forum_id and p.poster_id = u.user_id and
      f.cat_id = c.cat_id and p.post_id = pt.post_id and
      post_time > " . $code . " and f.forum_id in (" . $forum_list . ") 
      order by c.cat_order, f.forum_order, t.topic_title, post_time";

   // Uncomment next line to see SQL used
   // $msg .= "**DEBUG**\r\n" . $sql2 . "\r\n**DEBUG**\r\n";

   if ( !($result2 = $db->sql_query($sql2)))
     {
       message_die(GENERAL_ERROR, 'Unable to execute retrieve message summary for user', '', __LINE__, __FILE__, $sql);
     }

   // Format all the mail for this user

   $last_forum = "";
   $last_topic = "";
   $msg_count = 0;

   while ($row2 = $db->sql_fetchrow($result2)) {

      // Show name of forum only if it changes
      if ($row2['forum_name'] <> $last_forum) { 
        if ($html) {
           if ($last_forum <> "")
              $msg .= "</table>\r\n";
           $msg .= "<h2>" . $lang['digest_forum'] . "<a href=\"" . $siteURL . "viewforum." . $phpEx . "?" . POST_FORUM_URL . "=" . $row2['forum_id'] . "\">" . $row2['forum_name'] . "</a></h2>\r\n";
        }
        else
           $msg .= "\r\n<<<< " . $lang['digest_forum'] . " " . $row2['forum_name'] . 
              ", " . $siteURL . "viewforum." . $phpEx . "?" . POST_FORUM_URL . "=" . $row2['forum_id'] . " >>>>\r\n";
      }

      // Show name of topic only if it changes
      if ($row2['topic_title'] <> $last_topic) {
        if ($html) {
           if ($last_topic <> "")
              $msg .= "</table>\r\n";
           $msg .= "<h3>" . $lang['digest_topic'] . "<a href=\"" . $siteURL . "viewtopic." . $phpEx . "?" . POST_TOPIC_URL . "=" . $row2['topic_id'] . "\">" . $row2['topic_title'] . "</a></h3>\r\n";
           $msg .= "<table border=\"1\">\r\n";
           $msg .= "<tr>\r\n";
           if ($show_text)
              $msg .= "<th>" . $lang['digest_link'] . "</th><th>" . $lang['digest_post_time'] . "</th><th>" . $lang['digest_author'] . "</th><th>" . $lang['digest_message_excerpt'] . "</th></tr>\r\n";
           else
              $msg .= "<th>" . $lang['digest_link'] . "</th><th>" . $lang['digest_post_time'] . "</th><th>" . $lang['digest_author'] . "</th>\r\n";

           $msg .= "</tr>\r\n"; 
        }
        else 
           $msg .= "\r\n<< " . $lang['digest_topic'] . " " . $row2['topic_title'] . 
              ", " . $siteURL . "viewtopic." . $phpEx. "?" . POST_TOPIC_URL . "=" . $row2['topic_id'] . " >>\r\n\r\n"; 
        $last_topic = $row2['topic_title'];
      }

      // Show message information
      if ($html) {
         $msg .= "<tr>\r\n"; 
         $msg .= "<td><a href=\"" . $siteURL . "viewtopic." . $phpEx . "?" . POST_POST_URL . "=" . $row2['post_id'] . "#" . $row2['post_id'] . "\">" . $row2['post_id'] . "</a></td>";
         $msg .= "<td>" . $row2['Display Time'] . " " . date("T",$row2['post_time']) . "</td>";
         $msg .= "<td>" . $row2['Posted by'] . "</td>";
         if ($show_text)
            $msg .= "<td>" . $row2['Post Text'] . "</td>\r\n";
         $msg .= "</tr>\r\n"; 
      }
      else {
         $msg .= $lang['digest_posted_by'] . $row2['Posted by'] . $lang['digest_posted_at'] . $row2['Display Time'] . " " . date("T",$row2['post_time']) .
            ", " . $siteURL . "viewtopic." . $phpEx . "?" . POST_POST_URL . "=" . $row2['post_id'] . "#" . $row2['post_id'] . "\r\n";

         // If requested to show the message text
         if ($show_text) {
            if (strlen($row2['Post Text']) < ($extract_size + 3))
               $msg .= $row2['Post Text'] . "\r\n";
            else
               $msg .= $lang['digest_message_excerpt'] . ": " . $row2['Post Text'] . "\r\n";
            $msg .= "\r\n------------------------------\r\n"; 
         }
      }

      // If the forum has changed, note the change
      if ($row2['forum_name'] <> $last_forum) 
         $last_forum = $row2['forum_name'];

      $msg_count++;
   }

   $db->sql_freeresult($result2);

   if ($html) 
      $msg .= "</table>\r\n";

   if ($msg_count == 0)
      $msg .= $parastart . $lang['digest_no_new_messages'] . $paraend;

   // Send the email if there are messages or if user selected to send email anyhow
   if ($msg_count > 0 || $row['send_on_no_messages'] == 'YES') {

      if (!(is_object($emailer)))
         $emailer = new emailer($board_config['smtp_delivery']);

      if ($html) {

         $emailer->use_template('mail_digests_html',$userdata['user_lang']);

         // Apply a style sheet if requested for HTML digest. If no style sheet is wanted
         // then the link tag pointing to the style sheet is not displayed. A custom style
         // sheet gets first priority.

         if ($link_tag_unset) {

            $stylesheet = "";
   	      if ($digest_use_custom_stylesheet)
               $stylesheet = $digest_custom_stylesheet_path;
            elseif ($digest_use_default_stylesheet) {

               // Get the default style sheet to apply to the HTML email
         	   $sql2 = "SELECT style_name, head_stylesheet
         		   FROM " . THEMES_TABLE . "
         		   WHERE themes_id = " . $board_config['default_style'];
         	   if ( !($result2 = $db->sql_query($sql2)) )
         	   {
         		   message_die(CRITICAL_ERROR, 'Could not query database for theme info');
         	   }
               $row2 = $db->sql_fetchrow($result2);
               $stylesheet = "templates/" . $row2['style_name'] . "/" . $row2['head_stylesheet'];
               $db->sql_freeresult($result2);
            }

            if ($stylesheet <> "") 
   		   $link_tag = "<link rel=\"stylesheet\" type=\"text/css\" href=\"" . $siteURL . $stylesheet . "\" />";

            $link_tag_unset = false;

         }

      }

      else
         $emailer->use_template('mail_digests_text',$userdata['user_lang']);

      $emailer->extra_headers("From: " . $lang['digest_from_text_name'] . " <" . $lang['digest_from_email_address'] . ">\n");

      if ($html) {
         $emailer->extra_headers("MIME-Version: 1.0");
         $emailer->extra_headers("Content-type: text/html; charset=iso-8859-1");
      }
      else
         $emailer->extra_headers("Content-Type: text/plain; charset=us-ascii");

      $emailer->email_address($to);
      $emailer->set_subject($lang['digest_subject_line']);
      $emailer->assign_vars(array(
         'BOARD_URL' => $siteURL,
         'LINK' => $link_tag,
         'L_SITENAME' => $board_config['sitename'],
         'L_SALUTATION' => $lang['digest_salutation'],
         'SALUTATION' => $row['username'],
         'L_DIGEST_OPTIONS' => $lang['digest_your_digest_options'],
         'L_INTRODUCTION' => $lang['digest_introduction'],
         'L_FORMAT' => $lang['digest_format_short'],
         'FORMAT' => $row['format'],
         'L_MESSAGE_TEXT' => $lang['digest_show_message_text'],
         'MESSAGE_TEXT' => $row['show_text'],
         'L_MY_MESSAGES' => $lang['digest_show_my_messages'],
         'MY_MESSAGES' => $row['show_mine'],
         'L_FREQUENCY' => $lang['digest_frequency'],
         'FREQUENCY' => $row['digest_type'],
         'L_NEW_MESSAGES' => $lang['digest_show_only_new_messages'],
         'NEW_MESSAGES' => $row['new_only'],
         'L_SEND_DIGEST' => $lang['digest_send_if_no_new_messages'],
         'SEND_DIGEST' => $row['send_on_no_messages'],
         'L_SEND_TIME' => $lang['digest_hour_to_send_short'],
         'SEND_TIME' => date('g A', mktime($row['send_hour'])),
         'DIGEST_CONTENT' => $msg,
         'DISCLAIMER' => $lang['digest_disclaimer']));
      $emailer->send($html);
      $emailer->reset();

   }

}

$db->sql_freeresult($result);

?>